/**
 * RtfHelper.java
 * eu.gronos.kostenrechner.controller (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.files;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.io.BufferedOutputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTextPane;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.StyledDocument;
import javax.swing.text.rtf.RTFEditorKit;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;
import eu.gronos.kostenrechner.view.result.RtfFileChooser;

/**
 * Die Klasse stellt Methoden bereit, um RTF-Daten in die Zwischenablage zu
 * kopieren: {@link #kopiereRtfInZwischenablage()}, und in eine Datei zu
 * speichern: {@link #speichereRTF()}
 *
 * @author Peter Schuster (setrok)
 * @date 15.08.2018
 *
 */
public class RtfDocHelper implements ClipboardOwner {
	private final RTFEditorKit editorKit;
	private final StyledDocument doc;

	/**
	 * Der Konstruktor braucht aus der GUI das EditorKit und das StyledDocument
	 * 
	 * @param editorKit es muss schon ein {@link RTFEditorKit} sein!
	 * @param doc       ein {@link StyledDocument}
	 */
	public RtfDocHelper(RTFEditorKit editorKit, StyledDocument doc) {
		this.editorKit = editorKit;
		this.doc = doc;
	}

	/**
	 * Die Methode fügt den Text der JTextPane in die Zwischenablage ein. Versucht
	 * das jetzt mit text/rtf!
	 * 
	 */
	public void kopiereRtfInZwischenablage() {
		// StyledDocument doc = getDocument();
		Clipboard zwischenAblage = Toolkit.getDefaultToolkit().getSystemClipboard();
		DataFlavor[] flavors = null;
		String[] inhalte = null;
		Transferable rtft = null;
		try {
			// FIXME: auch hier funktioniert der Zeichensatz nicht.
			inhalte = new String[] { getRtfAsString(), getAllText() };
			flavors = new DataFlavor[] { new DataFlavor("text/rtf", "Rich Formatted Text"),
					new DataFlavor("text/plain", "Plain Text") };
			rtft = new RtfDataTransfer(flavors, inhalte);
			zwischenAblage.setContents(rtft, this);
		} catch (BadLocationException | UnsupportedEncodingException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
			// e.printStackTrace();
		}
	}

	/**
	 * 
	 * @see java.awt.datatransfer.ClipboardOwner#lostOwnership(java.awt.datatransfer.Clipboard,
	 *      java.awt.datatransfer.Transferable)
	 */
	@Override
	public void lostOwnership(Clipboard clipboard, Transferable contents) {
		// System.out.println
		Kostenrechner.getLogger()
				.info(String.format("Och nö, gehört mir doch die Zwischeablage (%s) nicht mehr!", clipboard.getName()));
	}

	/**
	 * Die Methode speichereRTF dient dazu, den Inhalt der JTextPane als RTF zu
	 * speichern. Die Variante ohne Parameter soll vom ActionListener der
	 * Schaltfläche selbst aufgerufen werden. Sie ruft nacheinander den
	 * Dateiauswahldialog auf, überprüft ob die Datei schon existiert und ruft dann
	 * die eigentliche Methode zum Speichern auf.
	 * 
	 * Write content from a document to the given stream in a format appropriate for
	 * this kind of content handler.
	 * 
	 */
	public boolean speichereRTF() {
		boolean zwischen = false;
		JFileChooser dateiAuswahl = new RtfFileChooser();
		int option = dateiAuswahl.showSaveDialog(dateiAuswahl);
		if (option == JFileChooser.APPROVE_OPTION) {
			File datei = dateiAuswahl.getSelectedFile().getAbsoluteFile();
			if (!datei.exists())
				zwischen = speichereRTF(datei);
			else {
				int bestaetigung = JOptionPane.showConfirmDialog(dateiAuswahl,
						"Die Datei existiert schon. \n\nFortfahren?");
				if (bestaetigung == JOptionPane.OK_OPTION)
					zwischen = speichereRTF(datei);
				else
					JOptionPane.showMessageDialog(dateiAuswahl, "Die Datei wurde nicht gespeichert.");
			}
		}
		return zwischen;
	}

	/**
	 * Die Methode schreibt den RTF-Inhalt des StyledDocument in einen
	 * ByteArrayOutputStream.
	 * 
	 * @return den Inhalt des ByteArrayOutputStream mittels toString()
	 * @throws UnsupportedEncodingException
	 */
	private String getRtfAsString() throws UnsupportedEncodingException {
		ByteArrayOutputStream bos = null;
		try {
			bos = new ByteArrayOutputStream(doc.getLength());
			editorKit.write(bos, doc, doc.getStartPosition().getOffset(), doc.getLength());
		} catch (IOException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} catch (BadLocationException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} finally {
			// der ByteArrayOutputStream muss nicht geschlossen werden
		}
		return bos != null ? bos.toString("UTF-8") : null;
	}

	/**
	 * Die Methode speichereRTF dient dazu, die <code>datei</code> wirklich zu
	 * speichern. Dazu wird
	 * {@link javax.swing.text.rtf.RTFEditorKit#write(OutputStream,Document,int,int)}
	 * benutzt.
	 * 
	 * @param datei die Datei, in die gespeichert werden soll
	 * @return hat's geklappt, dann true
	 * 
	 * @url "http://stackoverflow.com/questions/2725141/java-jtextpane-rtf-save"
	 * 
	 * @see javax.swing.text.rtf.RTFEditorKit#write(OutputStream,Document,int,int)
	 */
	private boolean speichereRTF(File datei) {
		// TODO Probleme mit dem Zeichensatz
		boolean zwischen = false;
		// StyledDocument doc = tenorDialog.getDocument();
		BufferedOutputStream bos = null;
		FileOutputStream fos = null;
		try {
			fos = new FileOutputStream(datei);
			bos = new BufferedOutputStream(fos);
			editorKit.write(bos, doc, doc.getStartPosition().getOffset(), doc.getLength());
			zwischen = true;
		} catch (FileNotFoundException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} catch (IOException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} catch (BadLocationException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e);
		} finally {
			if (bos != null)
				try {
					bos.close();
				} catch (IOException e) {
				}
			if (fos != null)
				try {
					fos.close();
				} catch (IOException e) {
				}
		}
		return zwischen;
	}

	/**
	 * Die Methode holt sich dem vollständigen Text der JTextPane
	 * 
	 * @return den gesamten Text der {@link JTextPane} als {@link String}
	 * @throws BadLocationException
	 */
	private String getAllText() throws BadLocationException {
		return doc.getText(doc.getStartPosition().getOffset(), doc.getLength());
	}

}
