/**
 * GebuehrenTatbestandHandler.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.awt.event.InputEvent;
import java.awt.event.MouseEvent;
import java.io.IOException;

import javax.swing.DropMode;
import javax.swing.JComponent;
import javax.swing.JTable;
import javax.swing.TransferHandler;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.AllzweckHorcher;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenSatzTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestandTableModel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * TransferHandler zum Weitergeben von GebührenTatbeständen
 * 
 * @author Peter Felix Schuster (SchusterP2)
 * @date 04.08.2014
 */
public class GebuehrenTransferHandler extends TransferHandler {

	private static final long serialVersionUID = -35313196581225380L;
	/**
	 * Der AllzweckHorcher mit dem Namen DRAG_HORCHER markiert beim Drag'n'Drop die
	 * Zeile über der sich der Mauszeiger befindet und stößt dann
	 * {@link javax.swing.TransferHandler#exportAsDrag(JComponent, InputEvent, int)
	 * exportAsDrag} an.
	 * 
	 * @see "http://stackoverflow.com/questions/638807/how-do-i-drag-and-drop-a-row-in-a-jtable"
	 *      How do I drag and drop a row in a JTable?
	 */
	private static final AllzweckHorcher DRAG_HORCHER = new AllzweckHorcher() {

		@Override
		public void mouseDragged(MouseEvent me) {
			me.consume();
			JTable table = (JTable) me.getSource();
			int row = table.rowAtPoint(me.getPoint());
			if (row > -1) {
				table.changeSelection(row, -1, false, false);
				TransferHandler handler = table.getTransferHandler();
				handler.exportAsDrag(table, me, TransferHandler.MOVE);
			} else {
				System.out.println("mouseDragged(MouseEvent me): keine Zeile markiert/markierbar.");
			}
		}
	};

	/**
	 * Transferable für {@link GebuehrenSatzTatbestand GebührenTatbeständen}
	 * 
	 * @author Peter Felix Schuster (SchusterP2)
	 * @date 04.08.2014
	 */
	static class GebuehrenTransferable implements Transferable {

		final private GebuehrenTatbestand gt;
		/**
		 * Der DataFlavor mit dem Namen GEBUEHREN_FLAVOR speichert einen
		 * {@link java.awt.datatransfer.DataFlavor DataFlavor} zum Umgang mit
		 * {@link GebuehrenSatzTatbestand GebührenTatbeständen}. Dazu nutzt er
		 * {@link DataFlavor.javaJVMLocalObjectMimeType}, um die Übergabe als nur für
		 * programminterne zu kennzeichnen.
		 * 
		 * @see java.awt.datatransfer.DataFlavor#javaJVMLocalObjectMimeType
		 */
		static final DataFlavor GEBUEHREN_FLAVOR = getDataFlavor();

		private static DataFlavor getDataFlavor() {
			try {
				return new DataFlavor(
						DataFlavor.javaJVMLocalObjectMimeType + ";class=eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand");
			} catch (ClassNotFoundException e) {
				FehlerDialog.zeigeFehler("Klasse nicht gefunden",
						String.format("Klasse nicht gefunden: GebuehrenTatbestand (%s)", e.getLocalizedMessage()), e);
				return null;
			}
		}

		/**
		 * Erzeugt einen neuen Transferable für den übergebenen
		 * {@link GebuehrenSatzTatbestand}.
		 * 
		 * @param gt der zu übertragende GebührenTatbestand
		 * 
		 */
		public GebuehrenTransferable(GebuehrenTatbestand gt) {
			super();
			this.gt = gt;
		}

		/**
		 * Returns an object which represents the data to be transferred. The class of
		 * the object returned is defined by the representation class of the flavor.
		 * 
		 * @param flavor the requested flavor for the data
		 * @return einen {@link GebuehrenSatzTatbestand}
		 * @throws UnsupportedFlavorException if the requested data flavor is not
		 *                                    supported.
		 * @throws IOException                if the data is no longer available in the
		 *                                    requested flavor.
		 * 
		 * @see java.awt.datatransfer.Transferable#getTransferData(java.awt.datatransfer.DataFlavor)
		 */
		@Override
		public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException {
			if (isDataFlavorSupported(flavor))
				return getGt();
			else
				throw new UnsupportedFlavorException(flavor);
			// return null;
		}

		/**
		 * Returns an array of DataFlavor objects indicating the flavors the data can be
		 * provided in. The array should be ordered according to preference for
		 * providing the data (from most richly descriptive to least descriptive).
		 * 
		 * @return an array of data flavors in which this data can be transferred
		 * 
		 * @see java.awt.datatransfer.Transferable#getTransferDataFlavors()
		 */
		@Override
		public DataFlavor[] getTransferDataFlavors() {
			return new DataFlavor[] { GEBUEHREN_FLAVOR };
		}

		/**
		 * Returns whether or not the specified data flavor is supported for this
		 * object.
		 * 
		 * @param flavor the requested flavor for the data
		 * @return boolean indicating whether or not the data flavor is supported
		 * 
		 * @see java.awt.datatransfer.Transferable#isDataFlavorSupported(java.awt.datatransfer.DataFlavor)
		 */
		@Override
		public boolean isDataFlavorSupported(DataFlavor flavor) {
			return GEBUEHREN_FLAVOR.equals(flavor);
		}

		/**
		 * @return the gt
		 */
		private GebuehrenTatbestand getGt() {
			return gt;
		}

	}

	/**
	 * Konstruktor: Erzeugt einen TransferHandler zum Weitergeben von
	 * GebührenTatbeständen und bereitet die JTable schonend darauf vor.
	 * 
	 * @param table die JTable, für die der TransferHandler tätig werden soll. Diese
	 *              wird nicht gespeichert, sondern nur Drag'n'Drop-bereit
	 *              geschaltet.
	 * 
	 */
	public GebuehrenTransferHandler(JTable table) {
		super();
		table.setDragEnabled(true);
		table.setDropMode(DropMode.ON_OR_INSERT);
		table.setFillsViewportHeight(true);
		table.addMouseMotionListener(DRAG_HORCHER);
	}

	/**
	 * This method is called repeatedly during a drag gesture and returns true if
	 * the area below the cursor can accept the transfer, or false if the transfer
	 * will be rejected. For example, if a user drags a color over a component that
	 * accepts only text, the canImport method for that component's TransferHandler
	 * should return false.
	 * 
	 * @param support the object containing the details of the transfer, not null.
	 * @return true if the import can happen, false otherwise
	 * 
	 * @see javax.swing.TransferHandler#canImport(javax.swing.TransferHandler.TransferSupport)
	 */
	@Override
	public boolean canImport(TransferSupport support) {
		if (support.isDrop())
			support.setShowDropLocation(false);
		if (!support.isDataFlavorSupported(GebuehrenTransferable.GEBUEHREN_FLAVOR)) {
			// System.out.println
			Kostenrechner.getLogger().info(
					String.format("Flavor not supported: %s!", GebuehrenTransferable.GEBUEHREN_FLAVOR.toString()));
			return false;
		}
		return true;
	}

	/**
	 * Causes the Swing drag support to be initiated. This is called by the various
	 * UI implementations in the javax.swing.plaf.basic package if the dragEnabled
	 * property is set on the component. This can be called by custom UI
	 * implementations to use the Swing drag support. This method can also be called
	 * by a Swing extension written as a subclass of JComponent to take advantage of
	 * the Swing drag support. The transfer will not necessarily have been completed
	 * at the return of this call (i.e. the call does not block waiting for the
	 * drop). The transfer will take place through the Swing implementation of the
	 * java.awt.dnd mechanism, requiring no further effort from the developer. The
	 * exportDone method will be called when the transfer has completed.
	 * 
	 * @param comp   the component holding the data to be transferred; provided to
	 *               enable sharing of TransferHandlers
	 * @param ie     the event that triggered the transfer
	 * @param action the transfer action initially requested; either COPY, MOVE or
	 *               LINK; the DnD system may change the action used during the
	 *               course of the drag operation
	 * 
	 * @see javax.swing.TransferHandler#exportAsDrag(javax.swing.JComponent,
	 *      java.awt.event.InputEvent, int)
	 */
	@Override
	public void exportAsDrag(JComponent comp, InputEvent ie, int action) {
		super.exportAsDrag(comp, ie, action);
	}

	/**
	 * Creates a Transferable to use as the source for a data transfer. Returns the
	 * representation of the data to be transferred, or null if the component's
	 * property is null
	 * 
	 * @param c the component holding the data to be transferred; provided to enable
	 *          sharing of TransferHandlers
	 * @return
	 * 
	 * @see javax.swing.TransferHandler#createTransferable(javax.swing.JComponent)
	 */
	@Override
	protected Transferable createTransferable(JComponent c) {
		if (isJTableValidSource(c)) {
			JTable t = (JTable) c;
			GebuehrenTatbestandTableModel gttm = (GebuehrenTatbestandTableModel) t.getModel();
			return new GebuehrenTransferable(gttm.getRow(t.getSelectedRow()));
		} else
			return null;
	}

	/**
	 * Prüft, ob die JComponent eine taugliche Quelle von GebührenTatbeständen ist.
	 * 
	 * @param comp the component holding the data to be transferred; provided to
	 *             enable sharing of TransferHandlers
	 * @return true, wenn {@link GebuehrenTransferHandler#isJTableValid(JComponent)
	 *         isJTableValid} und zusätzlich eine Zeile markiert ist.
	 */
	private boolean isJTableValidSource(JComponent comp) {
		return isJTableValid(comp) && ((JTable) comp).getSelectedRow() >= 0;
	}

	/**
	 * Prüft, ob die JComponent fähig ist, GebührenTatbestände abzugeben oder
	 * aufzunehmen
	 * 
	 * @param comp the component; provided to enable sharing of TransferHandlers
	 * @return true, comp eine JTable != null und das TableModel ein
	 *         GebuehrenTatbestandTableModel ist.
	 */
	private boolean isJTableValid(JComponent comp) {
		return comp != null && comp instanceof JTable
				&& ((JTable) comp).getModel() instanceof GebuehrenTatbestandTableModel;
	}

	/**
	 * Invoked after data has been exported. This method should remove the data that
	 * was transferred if the action was MOVE. This method is implemented to do
	 * nothing since MOVE is not a supported action of this implementation
	 * (getSourceActions does not include MOVE).
	 * 
	 * 
	 * @param source the component that was the source of the data
	 * @param data   The data that was transferred or possibly null if the action is
	 *               NONE.
	 * @param action the actual action that was performed
	 * 
	 * @see javax.swing.TransferHandler#exportDone(javax.swing.JComponent,
	 *      java.awt.datatransfer.Transferable, int)
	 */
	@Override
	protected void exportDone(JComponent source, Transferable data, int action) {
		if ((isJTableValidSource(source) && action == MOVE))/*  */ {
			JTable t = (JTable) source;
			((GebuehrenTatbestandTableModel) t.getModel()).removeRow(t.getSelectedRow());
		}
	}

	/**
	 * Returns the type of transfer actions supported by the source; any bitwise-OR
	 * combination of COPY, MOVE and LINK. Some models are not mutable, so a
	 * transfer operation of MOVE should not be advertised in that case. Returning
	 * NONE disables transfers from the component.
	 * 
	 * Gibt TransferHandler.MOVE zurück, wenn es sich um eine JTable mit einem
	 * {@link GebuehrenTatbestandTableModel} als Model handelt. Sonst NONE.
	 * 
	 * @param comp the component holding the data to be transferred; provided to
	 *             enable sharing of TransferHandlers
	 * @return
	 * 
	 * @see javax.swing.TransferHandler#getSourceActions(javax.swing.JComponent)
	 */
	@Override
	public int getSourceActions(JComponent comp) {
		return MOVE;
	}

	/**
	 * This method is called on a successful drop (or paste) and initiates the
	 * transfer of data to the target component. This method returns true if the
	 * import was successful and false otherwise.
	 * 
	 * Causes a transfer to occur from a clipboard or a drag and drop operation. The
	 * Transferable to be imported and the component to transfer to are contained
	 * within the TransferSupport. While the drag and drop implementation calls
	 * canImport to determine the suitability of a transfer before calling this
	 * method, the implementation of paste does not. As such, it cannot be assumed
	 * that the transfer is acceptable upon a call to this method for paste. It is
	 * recommended that canImport be explicitly called to cover this case.
	 * 
	 * Note: The TransferSupport object passed to this method is only valid for the
	 * duration of the method call. It is undefined what values it may contain after
	 * this method returns.
	 * 
	 * @param support the object containing the details of the transfer, not null.
	 * @return true if the data was inserted into the component, false otherwise
	 * @throws NullPointerException if <code>support</code> is null
	 * 
	 * @see javax.swing.TransferHandler#importData(javax.swing.TransferHandler.TransferSupport)
	 */
	@Override
	public boolean importData(TransferSupport support) throws NullPointerException {
		if (support == null)
			throw new NullPointerException();
		if (!isJTableValid((JComponent) support.getComponent())) {
			return false;
		}
		if (!canImport(support)) {
			return false;
		}
		try {
			GebuehrenTatbestand gt = (GebuehrenTatbestand) support.getTransferable()
					.getTransferData(GebuehrenTransferable.GEBUEHREN_FLAVOR);
			JTable table = (JTable) support.getComponent();
			GebuehrenTatbestandTableModel gttm = (GebuehrenTatbestandTableModel) table.getModel();
			/*
			 * Herausfinden, wo fallen gelassen wurde. Vorher prüfen, ob isDrop, denn sonst
			 * kann getDropLocation einen Fehler werfen. Ansonsten oder wenn die
			 * DropLocation außerhalb des Zeilenbereichs der Table ist, auf -1 setzen.
			 */
			int dropPoint = -1;
			if (support.isDrop()) {
				dropPoint = table.rowAtPoint(support.getDropLocation().getDropPoint());
			}
			/*
			 * Wenn innerhalb des gültigen Zeilenbereichs fallen gelassen: Element an
			 * bestimmter Stelle einfügen.
			 */
			if (dropPoint > -1 && dropPoint < gttm.getRowCount()/* +2 */) {
				int selection = gttm.getAllValues().indexOf(gt);
				gttm.addRowAt(dropPoint, gt);
				/*
				 * Falls das bisherige Element in derselben Tabelle war (indexOf > -1), muss
				 * sichergestellt werden, dass das richtige Element markiert wird. Denn die
				 * markierte Zeile wird gelöscht.
				 */
				if (selection > -1) {
					if (dropPoint < selection) {
						selection++;
					}
				}
				table.changeSelection(selection, -1, false, false);
			} else {
				/*
				 * Wenn das Element außerhalb des gültigen Zeilenbereichs fallen gelassen wurde,
				 * wird es ans Ende gesetzt. Dann muss auf andere Weise sichergestellt werden,
				 * dass das richtige Element markiert bleibt (zum Verschieben muss es vorher
				 * markiert worden sein).
				 */
				boolean isDieselbeTabelle = gttm.getAllValues().contains(gt);
				gttm.addRow(gt);
				if (!isDieselbeTabelle)
					table.changeSelection(gttm.getRowCount() - 1, -1, false, false);
			}
			return true;
		} catch (Exception e) {
			FehlerDialog.zeigeFehler("Fehler beim Gebührenkopieren",
					String.format("Fehler (%s) beim Kopieren eines Gebührentatbestands (%s): %s",
							e.getClass().toString(), "", e.getLocalizedMessage()),
					e);
			// System.err.println(e.getLocalizedMessage());
			// e.printStackTrace();
			return false;
		}
	}

}
